@@ -1,25 +1,26 @@
-/************************************************************************
- FAUST Architecture File
- Copyright (C) 2003-2017 GRAME, Centre National de Creation Musicale
- ---------------------------------------------------------------------
- This Architecture section is free software; you can redistribute it
- and/or modify it under the terms of the GNU General Public License
- as published by the Free Software Foundation; either version 3 of
- the License, or (at your option) any later version.
- 
- This program is distributed in the hope that it will be useful,
- but WITHOUT ANY WARRANTY; without even the implied warranty of
- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- GNU General Public License for more details.
- 
- You should have received a copy of the GNU General Public License
- along with this program; If not, see <http://www.gnu.org/licenses/>.
- 
- EXCEPTION : As a special exception, you may create a larger work
- that contains this FAUST architecture section and distribute
- that work under terms of your choice, so long as this FAUST
- architecture section is not modified.
- ************************************************************************/
+/************************** BEGIN APIUI.h *****************************
+FAUST Architecture File
+Copyright (C) 2003-2022 GRAME, Centre National de Creation Musicale
+---------------------------------------------------------------------
+This program is free software; you can redistribute it and/or modify
+it under the terms of the GNU Lesser General Public License as published by
+the Free Software Foundation; either version 2.1 of the License, or
+(at your option) any later version.
+
+This program is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
+GNU Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public License
+along with this program; if not, write to the Free Software
+Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
+
+EXCEPTION : As a special exception, you may create a larger work
+that contains this FAUST architecture section and distribute
+that work under terms of your choice, so long as this FAUST
+architecture section is not modified.
+************************************************************************/
 
 #ifndef API_UI_H
 #define API_UI_H
@@ -27,7 +28,7 @@
 #include <sstream>
 #include <string>
 #include <vector>
-#include <iostream>
+#include <stdio.h>
 #include <map>
 
 #include "faust/gui/meta.h"
@@ -35,28 +36,46 @@
 #include "faust/gui/PathBuilder.h"
 #include "faust/gui/ValueConverter.h"
 
+typedef unsigned int uint;
+
 class APIUI : public PathBuilder, public Meta, public UI
 {
     public:
-    
         enum ItemType { kButton = 0, kCheckButton, kVSlider, kHSlider, kNumEntry, kHBargraph, kVBargraph };
-  
+        enum Type { kAcc = 0, kGyr = 1, kNoType };
+
     protected:
-    
-        enum { kLin = 0, kLog = 1, kExp = 2 };
-    
-        int fNumParameters;
-        std::vector<std::string> fPaths;
-        std::vector<std::string> fLabels;
-        std::map<std::string, int> fPathMap;
-        std::map<std::string, int> fLabelMap;
-        std::vector<ValueConverter*> fConversion;
-        std::vector<FAUSTFLOAT*> fZone;
-        std::vector<FAUSTFLOAT> fInit;
-        std::vector<FAUSTFLOAT> fMin;
-        std::vector<FAUSTFLOAT> fMax;
-        std::vector<FAUSTFLOAT> fStep;
-        std::vector<ItemType> fItemType;
+
+        enum Mapping { kLin = 0, kLog = 1, kExp = 2 };
+
+        struct Item {
+            std::string fLabel;
+            std::string fShortname;
+            std::string fPath;
+            ValueConverter* fConversion;
+            FAUSTFLOAT* fZone;
+            FAUSTFLOAT fInit;
+            FAUSTFLOAT fMin;
+            FAUSTFLOAT fMax;
+            FAUSTFLOAT fStep;
+            ItemType fItemType;
+            
+            Item(const std::string& label,
+                 const std::string& short_name,
+                 const std::string& path,
+                 ValueConverter* conversion,
+                 FAUSTFLOAT* zone,
+                 FAUSTFLOAT init,
+                 FAUSTFLOAT min,
+                 FAUSTFLOAT max,
+                 FAUSTFLOAT step,
+                 ItemType item_type)
+            :fLabel(label), fShortname(short_name), fPath(path), fConversion(conversion),
+            fZone(zone), fInit(init), fMin(min), fMax(max), fStep(step), fItemType(item_type)
+            {}
+        };
+        std::vector<Item> fItems;
+
         std::vector<std::map<std::string, std::string> > fMetaData;
         std::vector<ZoneControl*> fAcc[3];
         std::vector<ZoneControl*> fGyr[3];
@@ -76,42 +95,38 @@ class APIUI : public PathBuilder, public Meta, public UI
         std::string fCurrentColor;
         std::string fCurrentTooltip;
         std::map<std::string, std::string> fCurrentMetadata;
-    
+
         // Add a generic parameter
         virtual void addParameter(const char* label,
-                                FAUSTFLOAT* zone,
-                                FAUSTFLOAT init,
-                                FAUSTFLOAT min,
-                                FAUSTFLOAT max,
-                                FAUSTFLOAT step,
-                                ItemType type)
+                                  FAUSTFLOAT* zone,
+                                  FAUSTFLOAT init,
+                                  FAUSTFLOAT min,
+                                  FAUSTFLOAT max,
+                                  FAUSTFLOAT step,
+                                  ItemType type)
         {
             std::string path = buildPath(label);
-            fPathMap[path] = fLabelMap[label] = fNumParameters++;
-            fPaths.push_back(path);
-            fLabels.push_back(label);
-            fZone.push_back(zone);
-            fInit.push_back(init);
-            fMin.push_back(min);
-            fMax.push_back(max);
-            fStep.push_back(step);
-            fItemType.push_back(type);
-            
-            //handle scale metadata
+            fFullPaths.push_back(path);
+
+            // handle scale metadata
+            ValueConverter* converter = nullptr;
             switch (fCurrentScale) {
                 case kLin:
-                    fConversion.push_back(new LinearValueConverter(0, 1, min, max));
+                    converter = new LinearValueConverter(0, 1, min, max);
                     break;
                 case kLog:
-                    fConversion.push_back(new LogValueConverter(0, 1, min, max));
+                    converter = new LogValueConverter(0, 1, min, max);
                     break;
-                case kExp: fConversion.push_back(new ExpValueConverter(0, 1, min, max));
+                case kExp:
+                    converter = new ExpValueConverter(0, 1, min, max);
                     break;
             }
             fCurrentScale = kLin;
-            
+
+            fItems.push_back(Item(label, "", path, converter, zone, init, min, max, step, type));
+       
             if (fCurrentAcc.size() > 0 && fCurrentGyr.size() > 0) {
-                std::cerr << "warning : 'acc' and 'gyr' metadata used for the same " << label << " parameter !!\n";
+                fprintf(stderr, "warning : 'acc' and 'gyr' metadata used for the same %s parameter !!\n", label);
             }
 
             // handle acc metadata "...[acc : <axe> <curve> <amin> <amid> <amax>]..."
@@ -127,11 +142,11 @@ class APIUI : public PathBuilder, public Meta, public UI
                 {
                     fAcc[axe].push_back(new CurveZoneControl(zone, curve, amin, amid, amax, min, init, max));
                 } else {
-                    std::cerr << "incorrect acc metadata : " << fCurrentAcc << std::endl;
+                    fprintf(stderr, "incorrect acc metadata : %s \n", fCurrentAcc.c_str());
                 }
                 fCurrentAcc = "";
             }
-       
+
             // handle gyr metadata "...[gyr : <axe> <curve> <amin> <amid> <amax>]..."
             if (fCurrentGyr.size() > 0) {
                 std::istringstream iss(fCurrentGyr);
@@ -145,91 +160,91 @@ class APIUI : public PathBuilder, public Meta, public UI
                 {
                     fGyr[axe].push_back(new CurveZoneControl(zone, curve, amin, amid, amax, min, init, max));
                 } else {
-                    std::cerr << "incorrect gyr metadata : " << fCurrentGyr << std::endl;
+                    fprintf(stderr, "incorrect gyr metadata : %s \n", fCurrentGyr.c_str());
                 }
                 fCurrentGyr = "";
             }
-        
+
             // handle screencolor metadata "...[screencolor:red|green|blue|white]..."
             if (fCurrentColor.size() > 0) {
-                if ((fCurrentColor == "red") && (fRedReader == 0)) {
+                if ((fCurrentColor == "red") && (fRedReader == nullptr)) {
                     fRedReader = new ZoneReader(zone, min, max);
                     fHasScreenControl = true;
-                } else if ((fCurrentColor == "green") && (fGreenReader == 0)) {
+                } else if ((fCurrentColor == "green") && (fGreenReader == nullptr)) {
                     fGreenReader = new ZoneReader(zone, min, max);
                     fHasScreenControl = true;
-                } else if ((fCurrentColor == "blue") && (fBlueReader == 0)) {
+                } else if ((fCurrentColor == "blue") && (fBlueReader == nullptr)) {
                     fBlueReader = new ZoneReader(zone, min, max);
                     fHasScreenControl = true;
-                } else if ((fCurrentColor == "white") && (fRedReader == 0) && (fGreenReader == 0) && (fBlueReader == 0)) {
+                } else if ((fCurrentColor == "white") && (fRedReader == nullptr) && (fGreenReader == nullptr) && (fBlueReader == nullptr)) {
                     fRedReader = new ZoneReader(zone, min, max);
                     fGreenReader = new ZoneReader(zone, min, max);
                     fBlueReader = new ZoneReader(zone, min, max);
                     fHasScreenControl = true;
                 } else {
-                    std::cerr << "incorrect screencolor metadata : " << fCurrentColor << std::endl;
+                    fprintf(stderr, "incorrect screencolor metadata : %s \n", fCurrentColor.c_str());
                 }
             }
             fCurrentColor = "";
-            
+
             fMetaData.push_back(fCurrentMetadata);
             fCurrentMetadata.clear();
         }
 
         int getZoneIndex(std::vector<ZoneControl*>* table, int p, int val)
         {
-            FAUSTFLOAT* zone = fZone[p];
+            FAUSTFLOAT* zone = fItems[uint(p)].fZone;
             for (size_t i = 0; i < table[val].size(); i++) {
-                if (zone == table[val][i]->getZone()) return i;
+                if (zone == table[val][i]->getZone()) return int(i);
             }
             return -1;
         }
-    
+
         void setConverter(std::vector<ZoneControl*>* table, int p, int val, int curve, double amin, double amid, double amax)
         {
             int id1 = getZoneIndex(table, p, 0);
             int id2 = getZoneIndex(table, p, 1);
             int id3 = getZoneIndex(table, p, 2);
-            
+
             // Deactivates everywhere..
-            if (id1 != -1) table[0][id1]->setActive(false);
-            if (id2 != -1) table[1][id2]->setActive(false);
-            if (id3 != -1) table[2][id3]->setActive(false);
-            
+            if (id1 != -1) table[0][uint(id1)]->setActive(false);
+            if (id2 != -1) table[1][uint(id2)]->setActive(false);
+            if (id3 != -1) table[2][uint(id3)]->setActive(false);
+
             if (val == -1) { // Means: no more mapping...
                 // So stay all deactivated...
             } else {
                 int id4 = getZoneIndex(table, p, val);
                 if (id4 != -1) {
                     // Reactivate the one we edit...
-                    table[val][id4]->setMappingValues(curve, amin, amid, amax, fMin[p], fInit[p], fMax[p]);
-                    table[val][id4]->setActive(true);
+                  table[val][uint(id4)]->setMappingValues(curve, amin, amid, amax, fItems[uint(p)].fMin, fItems[uint(p)].fInit, fItems[uint(p)].fMax);
+                  table[val][uint(id4)]->setActive(true);
                 } else {
                     // Allocate a new CurveZoneControl which is 'active' by default
-                    FAUSTFLOAT* zone = fZone[p];
-                    table[val].push_back(new CurveZoneControl(zone, curve, amin, amid, amax, fMin[p], fInit[p], fMax[p]));
+                    FAUSTFLOAT* zone = fItems[uint(p)].fZone;
+                    table[val].push_back(new CurveZoneControl(zone, curve, amin, amid, amax, fItems[uint(p)].fMin, fItems[uint(p)].fInit, fItems[uint(p)].fMax));
                 }
             }
         }
-    
+
         void getConverter(std::vector<ZoneControl*>* table, int p, int& val, int& curve, double& amin, double& amid, double& amax)
         {
             int id1 = getZoneIndex(table, p, 0);
             int id2 = getZoneIndex(table, p, 1);
             int id3 = getZoneIndex(table, p, 2);
-            
+
             if (id1 != -1) {
                 val = 0;
-                curve = table[val][id1]->getCurve();
-                table[val][id1]->getMappingValues(amin, amid, amax);
+                curve = table[val][uint(id1)]->getCurve();
+                table[val][uint(id1)]->getMappingValues(amin, amid, amax);
             } else if (id2 != -1) {
                 val = 1;
-                curve = table[val][id2]->getCurve();
-                table[val][id2]->getMappingValues(amin, amid, amax);
+                curve = table[val][uint(id2)]->getCurve();
+                table[val][uint(id2)]->getMappingValues(amin, amid, amax);
             } else if (id3 != -1) {
                 val = 2;
-                curve = table[val][id3]->getCurve();
-                table[val][id3]->getMappingValues(amin, amid, amax);
+                curve = table[val][uint(id3)]->getCurve();
+                table[val][uint(id3)]->getMappingValues(amin, amid, amax);
             } else {
                 val = -1; // No mapping
                 curve = 0;
@@ -239,41 +254,40 @@ class APIUI : public PathBuilder, public Meta, public UI
             }
         }
 
-     public:
-    
-        enum Type { kAcc = 0, kGyr = 1, kNoType };
-   
-        APIUI() : fNumParameters(0), fHasScreenControl(false), fRedReader(0), fGreenReader(0), fBlueReader(0)
+    public:
+
+        APIUI() : fHasScreenControl(false), fRedReader(nullptr), fGreenReader(nullptr), fBlueReader(nullptr), fCurrentScale(kLin)
         {}
 
         virtual ~APIUI()
         {
-            std::vector<ValueConverter*>::iterator it1;
-            for (it1 = fConversion.begin(); it1 != fConversion.end(); it1++) {
-                delete(*it1);
-            }
-
-            std::vector<ZoneControl*>::iterator it2;
+            for (const auto& it : fItems) delete it.fConversion;
             for (int i = 0; i < 3; i++) {
-                for (it2 = fAcc[i].begin(); it2 != fAcc[i].end(); it2++) {
-                    delete(*it2);
-                }
-                for (it2 = fGyr[i].begin(); it2 != fGyr[i].end(); it2++) {
-                    delete(*it2);
-                }
+                for (const auto& it : fAcc[i]) delete it;
+                for (const auto& it : fGyr[i]) delete it;
             }
-            
             delete fRedReader;
             delete fGreenReader;
             delete fBlueReader;
         }
-    
+
         // -- widget's layouts
 
-        virtual void openTabBox(const char* label)          { fControlsLevel.push_back(label); }
-        virtual void openHorizontalBox(const char* label)   { fControlsLevel.push_back(label); }
-        virtual void openVerticalBox(const char* label)     { fControlsLevel.push_back(label); }
-        virtual void closeBox()                             { fControlsLevel.pop_back(); }
+        virtual void openTabBox(const char* label) { pushLabel(label); }
+        virtual void openHorizontalBox(const char* label) { pushLabel(label); }
+        virtual void openVerticalBox(const char* label) { pushLabel(label); }
+        virtual void closeBox()
+        {
+            if (popLabel()) {
+                // Shortnames can be computed when all fullnames are known
+                computeShortNames();
+                // Fill 'shortname' field for each item
+                for (const auto& it : fFull2Short) {
+                    int index = getParamIndex(it.first.c_str());
+                    fItems[index].fShortname = it.second;
+                }
+            }
+        }
 
         // -- active widgets
 
@@ -306,25 +320,25 @@ class APIUI : public PathBuilder, public Meta, public UI
 
         virtual void addHorizontalBargraph(const char* label, FAUSTFLOAT* zone, FAUSTFLOAT min, FAUSTFLOAT max)
         {
-            addParameter(label, zone, min, min, max, (max-min)/1000.0, kHBargraph);
+            addParameter(label, zone, min, min, max, (max-min)/1000.0f, kHBargraph);
         }
 
         virtual void addVerticalBargraph(const char* label, FAUSTFLOAT* zone, FAUSTFLOAT min, FAUSTFLOAT max)
         {
-            addParameter(label, zone, min, min, max, (max-min)/1000.0, kVBargraph);
+            addParameter(label, zone, min, min, max, (max-min)/1000.0f, kVBargraph);
         }
-    
+
         // -- soundfiles
-    
+
         virtual void addSoundfile(const char* label, const char* filename, Soundfile** sf_zone) {}
 
         // -- metadata declarations
 
         virtual void declare(FAUSTFLOAT* zone, const char* key, const char* val)
         {
-            // Keep matadata
+            // Keep metadata
             fCurrentMetadata[key] = val;
-            
+
             if (strcmp(key, "scale") == 0) {
                 if (strcmp(val, "log") == 0) {
                     fCurrentScale = kLog;
@@ -349,54 +363,75 @@ class APIUI : public PathBuilder, public Meta, public UI
         virtual void declare(const char* key, const char* val)
         {}
 
-		//-------------------------------------------------------------------------------
-		// Simple API part
-		//-------------------------------------------------------------------------------
-		int getParamsCount()				{ return fNumParameters; }
-        int getParamIndex(const char* path)
+        //-------------------------------------------------------------------------------
+        // Simple API part
+        //-------------------------------------------------------------------------------
+        int getParamsCount() { return int(fItems.size()); }
+
+        int getParamIndex(const char* path_aux)
         {
-            if (fPathMap.find(path) != fPathMap.end()) {
-                return fPathMap[path];
-            } else if (fLabelMap.find(path) != fLabelMap.end()) {
-                return fLabelMap[path];
-            } else {
-                return -1;
-            }
+            std::string path = std::string(path_aux);
+            auto it = find_if(fItems.begin(), fItems.end(),
+                              [=](const Item& it) { return (it.fLabel == path) || (it.fShortname == path) || (it.fPath == path); });
+            return (it != fItems.end()) ? int(it - fItems.begin()) : -1;
         }
-        const char* getParamAddress(int p)	{ return fPaths[p].c_str(); }
-        const char* getParamLabel(int p)	{ return fLabels[p].c_str(); }
+    
+        const char* getParamLabel(int p) { return fItems[uint(p)].fLabel.c_str(); }
+        const char* getParamShortname(int p) { return fItems[uint(p)].fShortname.c_str(); }
+        const char* getParamAddress(int p) { return fItems[uint(p)].fPath.c_str(); }
+    
         std::map<const char*, const char*> getMetadata(int p)
         {
             std::map<const char*, const char*> res;
-            std::map<std::string, std::string> metadata = fMetaData[p];
-            std::map<std::string, std::string>::iterator it;
-            for (it = metadata.begin(); it != metadata.end(); ++it) {
-                res[(*it).first.c_str()] = (*it).second.c_str();
+            std::map<std::string, std::string> metadata = fMetaData[uint(p)];
+            for (const auto& it : metadata) {
+                res[it.first.c_str()] = it.second.c_str();
             }
             return res;
         }
 
         const char* getMetadata(int p, const char* key)
         {
-            return (fMetaData[p].find(key) != fMetaData[p].end()) ? fMetaData[p][key].c_str() : "";
+            return (fMetaData[uint(p)].find(key) != fMetaData[uint(p)].end()) ? fMetaData[uint(p)][key].c_str() : "";
         }
-        FAUSTFLOAT getParamMin(int p)		{ return fMin[p]; }
-        FAUSTFLOAT getParamMax(int p)		{ return fMax[p]; }
-        FAUSTFLOAT getParamStep(int p)		{ return fStep[p]; }
-        FAUSTFLOAT getParamInit(int p)		{ return fInit[p]; }
+        FAUSTFLOAT getParamMin(int p) { return fItems[uint(p)].fMin; }
+        FAUSTFLOAT getParamMax(int p) { return fItems[uint(p)].fMax; }
+        FAUSTFLOAT getParamStep(int p) { return fItems[uint(p)].fStep; }
+        FAUSTFLOAT getParamInit(int p) { return fItems[uint(p)].fInit; }
 
-        FAUSTFLOAT* getParamZone(int p)         { return fZone[p]; }
-        FAUSTFLOAT getParamValue(int p)         { return *fZone[p]; }
-        void setParamValue(int p, FAUSTFLOAT v) { *fZone[p] = v; }
+        FAUSTFLOAT* getParamZone(int p) { return fItems[uint(p)].fZone; }
 
-        double getParamRatio(int p)         { return fConversion[p]->faust2ui(*fZone[p]); }
-        void setParamRatio(int p, double r) { *fZone[p] = fConversion[p]->ui2faust(r); }
+        FAUSTFLOAT getParamValue(int p) { return *fItems[uint(p)].fZone; }
+        FAUSTFLOAT getParamValue(const char* path)
+        {
+            int index = getParamIndex(path);
+            if (index >= 0) {
+                return getParamValue(index);
+            } else {
+                fprintf(stderr, "getParamValue : '%s' not found\n", (path == nullptr ? "NULL" : path));
+                return FAUSTFLOAT(0);
+            }
+        }
+
+        void setParamValue(int p, FAUSTFLOAT v) { *fItems[uint(p)].fZone = v; }
+        void setParamValue(const char* path, FAUSTFLOAT v)
+        {
+            int index = getParamIndex(path);
+            if (index >= 0) {
+                setParamValue(index, v);
+            } else {
+                fprintf(stderr, "setParamValue : '%s' not found\n", (path == nullptr ? "NULL" : path));
+            }
+        }
+
+        double getParamRatio(int p) { return fItems[uint(p)].fConversion->faust2ui(*fItems[uint(p)].fZone); }
+        void setParamRatio(int p, double r) { *fItems[uint(p)].fZone = FAUSTFLOAT(fItems[uint(p)].fConversion->ui2faust(r)); }
+
+        double value2ratio(int p, double r)    { return fItems[uint(p)].fConversion->faust2ui(r); }
+        double ratio2value(int p, double r)    { return fItems[uint(p)].fConversion->ui2faust(r); }
 
-        double value2ratio(int p, double r)	{ return fConversion[p]->faust2ui(r); }
-        double ratio2value(int p, double r)	{ return fConversion[p]->ui2faust(r); }
-    
         /**
-         * Return the control type (kAcc, kGyr, or -1) for a given parameter
+         * Return the control type (kAcc, kGyr, or -1) for a given parameter.
          *
          * @param p - the UI parameter index
          *
@@ -417,9 +452,9 @@ class APIUI : public PathBuilder, public Meta, public UI
             }
             return kNoType;
         }
-    
+
         /**
-         * Return the Item type (kButton = 0, kCheckButton, kVSlider, kHSlider, kNumEntry, kHBargraph, kVBargraph) for a given parameter
+         * Return the Item type (kButton = 0, kCheckButton, kVSlider, kHSlider, kNumEntry, kHBargraph, kVBargraph) for a given parameter.
          *
          * @param p - the UI parameter index
          *
@@ -427,11 +462,11 @@ class APIUI : public PathBuilder, public Meta, public UI
          */
         ItemType getParamItemType(int p)
         {
-            return fItemType[p];
+            return fItems[uint(p)].fItemType;
         }
-   
+
         /**
-         * Set a new value coming from an accelerometer, propagate it to all relevant float* zones.
+         * Set a new value coming from an accelerometer, propagate it to all relevant FAUSTFLOAT* zones.
          *
          * @param acc - 0 for X accelerometer, 1 for Y accelerometer, 2 for Z accelerometer
          * @param value - the new value
@@ -443,7 +478,7 @@ class APIUI : public PathBuilder, public Meta, public UI
                 fAcc[acc][i]->update(value);
             }
         }
-    
+
         /**
          * Used to edit accelerometer curves and mapping. Set curve and related mapping for a given UI parameter.
          *
@@ -459,12 +494,12 @@ class APIUI : public PathBuilder, public Meta, public UI
         {
             setConverter(fAcc, p, acc, curve, amin, amid, amax);
         }
-    
+
         /**
          * Used to edit gyroscope curves and mapping. Set curve and related mapping for a given UI parameter.
          *
          * @param p - the UI parameter index
-         * @param acc - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope (-1 means "no mapping")
+         * @param gyr - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope (-1 means "no mapping")
          * @param curve - between 0 and 3
          * @param amin - mapping 'min' point
          * @param amid - mapping 'middle' point
@@ -473,9 +508,9 @@ class APIUI : public PathBuilder, public Meta, public UI
          */
         void setGyrConverter(int p, int gyr, int curve, double amin, double amid, double amax)
         {
-             setConverter(fGyr, p, gyr, curve, amin, amid, amax);
+            setConverter(fGyr, p, gyr, curve, amin, amid, amax);
         }
-    
+
         /**
          * Used to edit accelerometer curves and mapping. Get curve and related mapping for a given UI parameter.
          *
@@ -507,9 +542,9 @@ class APIUI : public PathBuilder, public Meta, public UI
         {
             getConverter(fGyr, p, gyr, curve, amin, amid, amax);
         }
-    
+
         /**
-         * Set a new value coming from an gyroscope, propagate it to all relevant float* zones.
+         * Set a new value coming from an gyroscope, propagate it to all relevant FAUSTFLOAT* zones.
          *
          * @param gyr - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope
          * @param value - the new value
@@ -521,7 +556,31 @@ class APIUI : public PathBuilder, public Meta, public UI
                 fGyr[gyr][i]->update(value);
             }
         }
-   
+
+        /**
+         * Get the number of FAUSTFLOAT* zones controlled with the accelerometer.
+         *
+         * @param acc - 0 for X accelerometer, 1 for Y accelerometer, 2 for Z accelerometer
+         * @return the number of zones
+         *
+         */
+        int getAccCount(int acc)
+        {
+            return (acc >= 0 && acc < 3) ? int(fAcc[acc].size()) : 0;
+        }
+
+        /**
+         * Get the number of FAUSTFLOAT* zones controlled with the gyroscope.
+         *
+         * @param gyr - 0 for X gyroscope, 1 for Y gyroscope, 2 for Z gyroscope
+         * @param the number of zones
+         *
+         */
+        int getGyrCount(int gyr)
+        {
+            return (gyr >= 0 && gyr < 3) ? int(fGyr[gyr].size()) : 0;
+        }
+
         // getScreenColor() : -1 means no screen color control (no screencolor metadata found)
         // otherwise return 0x00RRGGBB a ready to use color
         int getScreenColor()
@@ -539,3 +598,4 @@ class APIUI : public PathBuilder, public Meta, public UI
 };
 
 #endif
+/**************************  END  APIUI.h **************************/
